// read SW1(RD9) and sound speaker (RB6)
// use TIMER2 interrupt
//
// PR2=   50;  // Fosc=29.48MHz/256/2/50   = 1.15kHz (Tosc= 868us)
// PR2=  100;  // Fosc=29.48MHz/256/2/100  =  576Hz  (Tosc=1.74ms)
// PR2= 1000;  // Fosc=29.48MHz/256/2/1000 = 57.6Hz  (Tosc=17.4ms)
// PR2=10000;  // Fosc=29.48MHz/256/2/10000= 5.76Hz  (Tosc= 174ms)
// PR2=65535;  // Fosc=29.48MHz/256/2/65535=0.879Hs  (Tosc=1.14s)

#include <p30F2012.h>        

//----------------------------------------------------------------------
// TIMER2 interrupt handler

void __attribute__ ((__interrupt__)) _T2Interrupt(void) {
  IFS0bits.T2IF=0;                // clear interrupt
  PORTBbits.RB6=~PORTBbits.RB6;   // toggle RB6 output
}

//----------------------------------------------------------------------

void main(void) {
  TRISBbits.TRISB6=0;             // set TRIS register, RB6 is output
  TRISDbits.TRISD9=1;             // set TRIS register, RD9 is input       

  INTCON2=0xF000;                 // use AIVT (Alternate Interrupt Vector Table)

  T2CON=0x0000;                   // clear TIMER2 control register
  TMR2 =0x0000;                   // clear TIMER2 counter register
  PR2  =50;                       // set TIMER2 period register (1.15kHz tone)
  IFS0 =0x0000;                   // clear all interrupt
  T2CON=0x8030;                   // start TIMER2 (presclaer 1:256)
  IEC0 =0x0040;                   // enable TIMER2 interrupt

  while(1) {                      // forever loop
    if (PORTDbits.RD9==1) {       // if SW1 is on
      IEC0=0x0040;                // enable TIMER2 interrupt 
    } else {
      IEC0=0x0000;                // disable TIMER2 interrupt
      PORTBbits.RB6=0;            // RB6 output is Lo (0V)
    }
  }
}